"use strict";

(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports);
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports);
    global.undefined = mod.exports;
  }
})(void 0, function (exports) {
  "use strict";

  Object.defineProperty(exports, "__esModule", {
    value: true
  });
  exports.default = sync;

  function sync(animationName) {
    var elements = new Set();
    var eventTime;
    var lastInterationTS = now();

    function animationStart(event) {
      if (event.animationName === animationName) {
        var el = event.target;
        var passed = now() - lastInterationTS;
        el.style.setProperty('animation-delay', "".concat(-passed, "ms"));
        elements.add(el);
      }
    }

    function animationIteration(event) {
      if (event.animationName === animationName) {
        elements.add(event.target);
        requestAnimationFrame(function (frameTime) {
          if (frameTime !== eventTime) {
            lastInterationTS = now();
            restart(elements);
          }

          eventTime = frameTime;
        });
      }
    }

    window.addEventListener('animationstart', animationStart, true);
    window.addEventListener('animationiteration', animationIteration, true);
    return {
      free: function free() {
        window.removeEventListener('animationstart', animationStart);
        window.removeEventListener('animationiteration', animationIteration);
        this.start();
        elements.clear();
      },
      start: function start() {
        elements.forEach(function (el) {
          if (validate(elements, el)) {
            el.style.removeProperty('animation');
          }
        });
      },
      stop: function stop() {
        elements.forEach(function (el) {
          if (validate(elements, el)) {
            el.style.setProperty('animation', 'none');
          }
        });
      },
      pause: function pause() {
        elements.forEach(function (el) {
          if (validate(elements, el)) {
            el.style.setProperty('animation-play-state', 'paused');
          }
        });
      }
    };
  }

  function restart(elements) {
    elements.forEach(function (el) {
      if (window.getComputedStyle(el).animationPlayState !== 'paused') {
        if (validate(elements, el)) {
          el.style.setProperty('animation', 'none');
        }
      }
    });
    requestAnimationFrame(function () {
      elements.forEach(function (el) {
        if (window.getComputedStyle(el).animationPlayState !== 'paused') {
          if (validate(elements, el)) {
            el.style.removeProperty('animation');
          }
        }
      });
    });
  }

  function now() {
    return new Date().getTime();
  }

  function validate(elements, el) {
    var isValid = document.body.contains(el);

    if (!isValid) {
      elements.delete(el);
    }

    return isValid;
  }

  module.exports = exports.default;
});
